from wnet.wnet_cpp import CWassersteinNetwork

class WassersteinNetwork(CWassersteinNetwork):
    def __init__(self, base_distribution, target_distributions, distance, max_distance=None):
        if max_distance is None:
            max_distance = CWassersteinNetwork.max_value()
        super().__init__(base_distribution, target_distributions, distance, max_distance)

    def subgraphs(self):
        return [SubgraphWrapper(self.get_subgraph(i)) for i in range(self.no_subgraphs())]


class SubgraphWrapper:
    def __init__(self, obj):
        self._obj = obj

    def __getattr__(self, name):
        return getattr(self._obj, name)

    def as_netowkrx(self):
        import networkx as nx
        G = nx.DiGraph()
        for node in self.get_nodes():
            G.add_node(node.get_id(), layer=node.layer(), type=node.type_str())
        for edge in self.get_edges():
            start = edge.get_start_node_id()
            end = edge.get_end_node_id()
            G.add_edge(start, end, capacity=edge.get_base_capacity(), weight=edge.get_cost())
        return G

    def show(self):
        import matplotlib.pyplot as plt
        import networkx as nx
        G = self.as_netowkrx()
        pos = nx.multipartite_layout(G, subset_key="layer")
        node_colors = []
        for _, data in G.nodes(data=True):
            if data["type"] == "source":
                node_colors.append("lightgreen")
            elif data["type"] == "sink":
                node_colors.append("lightcoral")
            elif data["type"] == "trash":
                node_colors.append("lightgray")
            else:
                node_colors.append("lightblue")
        edge_labels = {(u, v): f"cost: {d['weight']}\n capacity: {d['capacity']}" for u, v, d in G.edges(data=True)}
        nx.draw(G, pos, with_labels=True, node_color=node_colors, arrows=True)
        nx.draw_networkx_edge_labels(G, pos, edge_labels=edge_labels)
        plt.show()



import importlib.metadata
from pathlib import Path


__version__ = importlib.metadata.version("wnet")

def module_main():
    import argparse
    parser = argparse.ArgumentParser(description="WNet: A module for calculation of Wasserstein metric between distributions using network flow-based approach.")
    parser.add_argument("--version", "-v",action="version", version=__version__)
    parser.add_argument("--include", "-i", help="Print include path for C++ headers", action="store_true")

    args = parser.parse_args()

    if args.include:
        print(Path(__file__).parent / "cpp")
    else:
        parser.print_help()

if __name__ == "__main__":
    module_main()


from .wasserstein_network import WassersteinNetwork

def WassersteinDistance(distribution1, distribution2, distance):
    assert distribution1.sum_intensities == distribution2.sum_intensities, "Distributions must have the same total intensity"
    W = WassersteinNetwork(distribution1, [distribution2], distance, None)
    W.build()
    W.solve()
    return W.total_cost()

def TruncatedWassersteinDistance(distribution1, distribution2, distance, max_distance):
    assert distribution1.sum_intensities == distribution2.sum_intensities, "Distributions must have the same total intensity"
    W = WassersteinNetwork(distribution1, [distribution2], distance, max_distance)
    W.build()
    W.add_simple_trash(max_distance)
    W.solve()
    return W.total_cost()


import numpy as np
from functools import cached_property
from wnet.wnet_cpp import CDistribution

class Distribution(CDistribution):
    def __init__(self, positions, intensities):
        super().__init__(positions, intensities)

    def scaled(self, scale_factor):
        new_positions = self.positions
        new_intensities = self.intensities * scale_factor
        return Distribution(new_positions, new_intensities)

    @property
    def positions(self):
        return self.get_positions()

    @property
    def intensities(self):
        return self.get_intensities()

    @cached_property
    def sum_intensities(self):
        return np.sum(self.intensities)

def Distribution_1D(positions, intensities):
    if not isinstance(positions, np.ndarray):
        positions = np.array(positions)
    if not isinstance(intensities, np.ndarray):
        intensities = np.array(intensities)
    assert len(positions.shape) == 1
    assert len(intensities.shape) == 1
    assert positions.shape[0] == intensities.shape[0]
    return Distribution(positions[np.newaxis, :], intensities)



from wnet import Distribution, WassersteinNetwork
from wnet.distances import L1Distance, wrap_distance_function
import numpy as np

positions1 = np.array(
    [[0, 1, 5, 10],
     [0, 0, 0, 3]]
)
intensities1 = np.array([10, 5, 5, 5])

positions2 = np.array(
    [[1,10],
    [0, 0]])
intensities2 = np.array([20, 5])

S1 = Distribution(positions1, intensities1)
S2 = Distribution(positions2, intensities2)

W = WassersteinNetwork(S1, [S2], L1Distance(), 10)
W.add_simple_trash(15)
W.build()
W.solve()
print("Total cost:", W.total_cost())
print(W.subgraphs()[0].as_netowkrx())
print(W.flows_for_target(0))
W.subgraphs()[0].show()